/*
 *  ======== SPIFlash.c ========
 */
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>


/* Driver Header files */
#include <ti/drivers/UART.h>
#include <ti/drivers/SPI.h>

/* Example/Board Header files */
#include "Board.h"
#include "string.h"
#include <ti/drivers/spi/SPICC26XXDMA.h>
#include <ti/drivers/dma/UDMACC26XX.h>
#include <ti/drivers/pin/PINCC26XX.h>

#define FLASH_WRITE_ENABLE          0x06
#define FLASH_WRITE_DISABLE         0x04
#define FLASH_READ_STATUS_REG       0x05
#define FLASH_WRITE_STATUS_REG      0x01
#define FLASH_READ_DATA             0x03
#define FLASH_SECTOR_ERASE          0x20
#define FLASH_WRITE_DATA            0x02
#define FLASH_READ_MANUDEVID        0x90

static SPI_Handle   spiHandle;
static PIN_Handle hFlashPin = NULL;
static uint8_t devInfoBuf[3];

static void flashSelect(void){
	PIN_setOutputValue(hFlashPin,Board_SPI_FLASH_CS,Board_FLASH_CS_ON);
}

static void flashDeSelect(void){
	PIN_setOutputValue(hFlashPin,Board_SPI_FLASH_CS,Board_FLASH_CS_OFF);
}

static bool spi_open(uint32_t bitRate) {
	SPI_Params spiParams;
    /*  Configure SPI as master */
    SPI_Params_init(&spiParams);
    spiParams.bitRate = bitRate;
    spiParams.mode = SPI_MASTER;
    spiParams.transferMode = SPI_MODE_BLOCKING;

    /* Attempt to open SPI. */
    spiHandle = SPI_open(Board_SPI0, &spiParams);

    return spiHandle != NULL;
}

static bool spi_write(const uint8_t *buf, size_t len) {
    SPI_Transaction masterTransaction;

    masterTransaction.count  = len;
    masterTransaction.txBuf  = (void*)buf;
    masterTransaction.arg    = NULL;
    masterTransaction.rxBuf  = NULL;

    return SPI_transfer(spiHandle, &masterTransaction) ? 1 : 0;
}

static bool spi_read(uint8_t *buf, size_t len) {
    SPI_Transaction masterTransaction;

    masterTransaction.count = len;
    masterTransaction.rxBuf = buf;
    masterTransaction.txBuf = NULL;
    masterTransaction.arg = NULL;

    return SPI_transfer(spiHandle, &masterTransaction) ? 1 : 0;
}

static bool flash_writeEnable(void) {
	bool flag;
	const uint8_t comadd[] = {FLASH_WRITE_ENABLE};//!< 将指令“写使能”放入数组中
	flashSelect();
	flag = spi_write(comadd,sizeof(comadd));//!< 调用spi_write写入指令
	flashDeSelect();
	return flag;
}
static bool flash_getInfo(void) {
	bool flag;
	const uint8_t comadd[] = {FLASH_READ_MANUDEVID, 0x00, 0x00, 0x00};//!< 将读Manufacturer/Device ID指令以及地址放在数组中待写入
	flashSelect();//!< 选中使能Flash
	flag = spi_write(comadd, sizeof(comadd));//!< 调用spi_write写入指令和地址
	if(!flag) {
		flashDeSelect();
		return false;
	}
	flag = spi_read(devInfoBuf, sizeof(devInfoBuf));//!< 调用spi_read读出Manufacturer/Device ID
	flashDeSelect();//!< 失能Flash
	return flag;
}

/*
 *  ======== mainThread ========
 */
void *mainThread(void *arg0)
{   bool flag;
	UART_Params uartParams;
	UART_Handle uart;
	
	SPI_init();  // 初始化SPI driver
	flag = spi_open(1000000);//!< 打开SPI
	if(!flag) {
		while(1);
	}
	
	flag = flash_writeEnable();//!< Flash写使能
	if(!flag) {
		while(1);
	}
	
	flag = flash_getInfo();//!< Flash读出Manufacturer/Device ID信息
	if(!flag) {
		while(1);
	}
	
    UART_init();//!< 初始化UART

    /* Create a UART with data processing off. */
    UART_Params_init(&uartParams);
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.readDataMode = UART_DATA_BINARY;
    uartParams.readReturnMode = UART_RETURN_FULL;
    uartParams.readEcho = UART_ECHO_OFF;
    uartParams.baudRate = 115200;

    uart = UART_open(Board_UART0, &uartParams);
	if (uart == NULL) {
        /* UART_open() failed */
        while (1);
    }

    UART_write(uart, devInfoBuf, sizeof(devInfoBuf));
	while(1);
	
}


